# Author-OA Fakinlede
# Description-Linearly increasing spiral
# Creation Date: July 11, 2020
# Last Modification: July 11, 2020
# Based On: Interpipes Spline, Interpipes Offset
# Purpose:  1. To teach Python by Solid Modeling & Simulations Objects
#           2. Quick start to CFD and Design Analysis


import adsk.core, adsk.fusion, adsk.cam, traceback, math

try:
    app = adsk.core.Application.get()
    ui = app.userInterface
    ui.messageBox('Linear Spiral Creation')
    doc = app.documents.add(adsk.core.DocumentTypes.FusionDesignDocumentType)    
    design = app.activeProduct
    # Get the root component of the active design.
    rootComp = design.rootComponent
    # Create a new sketch on the xy plane{Symmetry plane of the model}.
    symPlane = rootComp.sketches.add(rootComp.xYConstructionPlane)
    # Convert diameter to cm
    diam = (100/10)
    
    # Create an object collection for the points in each spline.
    splPoints = adsk.core.ObjectCollection.create()
    startRange = 0 
    # Start of range to be evaluated.
    endRange = 2 * math.pi 
    # End of range to be evaluated.
    splinePoints = 7 
    # Number of points that create the splines.
    i = 0
    rad = diam/2
    while i <= splinePoints:
        t = startRange + ((endRange - startRange)/splinePoints) * i
        xCoord = ((rad + t /(2*math.pi)) * math.cos(t))
        yCoord = ((rad + t /(2*math.pi)) * math.sin(t))
        splPoints.add(adsk.core.Point3D.create(xCoord, yCoord, 0))
        i += 1
    # Generates the spline curve
    spl = symPlane.sketchCurves.sketchFittedSplines.add(splPoints)
    # Create sketch, on XZ plane,  
    # And draw two center circles at the of the spline
    begPlane = rootComp.sketches.add(rootComp.xZConstructionPlane)
    sketchCirclesBeg = begPlane.sketchCurves.sketchCircles
    centerPoint = adsk.core.Point3D.create(rad, 0, 0)
    circle1 = sketchCirclesBeg.addByCenterRadius(centerPoint, rad/4)
    circle2 = sketchCirclesBeg.addByCenterRadius(centerPoint, (rad/4)*(12/10))
    # Get the profile defined by the two circles. That is the first item 
    #   in the sketches container
    profBeg1A = begPlane.profiles.item(0)
    profBeg1B = begPlane.profiles.item(1)
    # Create another sketch, on the same XZ plane,  
    # And draw two center circles at the of the spline
    # Find it easier to locate profiles this way
    endPlane = rootComp.sketches.add(rootComp.xZConstructionPlane)
    sketchCirclesEnd = endPlane.sketchCurves.sketchCircles
    centerPoint = adsk.core.Point3D.create(rad*(12/10), 0, 0)
    circle1 = sketchCirclesEnd.addByCenterRadius(centerPoint, rad/2)
    circle2 = sketchCirclesEnd.addByCenterRadius(centerPoint, (rad/2)*(11.5/10))
    # Get the profile defined by the two circles. That is the first item 
    #   in the sketches container
    profEnd1A = endPlane.profiles.item(0)  
    profEnd1B = endPlane.profiles.item(1)   

    # Body-Creation Loft1   
    loft1 = rootComp.features.loftFeatures
    loftInput = loft1.createInput(adsk.fusion.FeatureOperations.NewBodyFeatureOperation)
    loftSectionsObj = loftInput.loftSections
    loftSectionsObj.add(profBeg1B)
    loftSectionsObj.add(profEnd1B)
    loftInput.centerLineOrRails.addCenterLine(spl)
    # Create loft feature
    loftOne = loft1.add(loftInput)
    # loftBody = loft1.bodies.item(0)
    # loftBody.name = "Joining Pipe" 
    #    
    # Create the extrusion container. Each new extrusion adds to it.
    extrudes = rootComp.features.extrudeFeatures
    # Vertical Hollow Cylinder: Create a symmetric extrusion that goes 120 mm from the profile plane.
    distance = adsk.core.ValueInput.createByReal(diam)
    extrudeOne = extrudes.addSimple(profEnd1B, distance, adsk.fusion.FeatureOperations.NewBodyFeatureOperation) 
    # Create the extrusion cut: Add it to the container
    #   
    distance = adsk.core.ValueInput.createByReal(diam)
    extrudeTwo = extrudes.addSimple(profEnd1A, distance, adsk.fusion.FeatureOperations.CutFeatureOperation) 
    # Create the extrusion: Add it to the container  
    # Body-Cutting Loft1   
    loft2 = rootComp.features.loftFeatures
    loftInput = loft2.createInput(adsk.fusion.FeatureOperations.CutFeatureOperation)
    loftSectionsObj = loftInput.loftSections
    loftSectionsObj.add(profBeg1A)
    loftSectionsObj.add(profEnd1A)
    loftInput.centerLineOrRails.addCenterLine(spl)
    # Create loft feature
    loftTwo = loft2.add(loftInput)
    # loftBody = loft1.bodies.item(0)
    # loftBody.name = "Joining Pipe" 
    #  
    # 
    # Create sketch, on XY plane, to make a vertical cylinder
    # And draw two center circles at the origin
    # For later designs, this origin will be 
    #   the point of meeting of intersecting objects
    sketchXY = rootComp.sketches.add(rootComp.xYConstructionPlane)
    sketchCirclesXY = sketchXY.sketchCurves.sketchCircles
    centerPoint = adsk.core.Point3D.create(0, 0, 0)
    circle1 = sketchCirclesXY.addByCenterRadius(centerPoint, diam/2)
    # Get the profile defined by the two circles. That is the first item 
    #   in the sketches container
    profXY = sketchXY.profiles.item(0)
      
# Error handeling
except:
    if ui:
        ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))